"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.translator = exports.cache = exports.jwtVerifier = exports.router = exports.errorLogger = void 0;
var tslib_1 = require("tslib");
var jsonwebtoken_1 = require("jsonwebtoken");
var mobx_i18n_1 = require("mobx-i18n");
var web_utility_1 = require("web-utility");
function errorLogger(context, next) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var error_1, status_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 3]);
                    return [4 /*yield*/, next()];
                case 1: return [2 /*return*/, _a.sent()];
                case 2:
                    error_1 = _a.sent();
                    console.error(error_1);
                    status_1 = error_1.status;
                    if (status_1 === 404)
                        return [2 /*return*/, { notFound: true, props: {} }];
                    throw error_1;
                case 3: return [2 /*return*/];
            }
        });
    });
}
exports.errorLogger = errorLogger;
function router(context, next) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var options, resolvedUrl, params, query, locales;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, next()];
                case 1:
                    options = (_a.sent()) || {}, resolvedUrl = context.resolvedUrl, params = context.params, query = context.query, locales = context.locales;
                    return [2 /*return*/, tslib_1.__assign(tslib_1.__assign({}, options), { props: tslib_1.__assign(tslib_1.__assign({}, ('props' in options ? options.props : {})), { route: JSON.parse(JSON.stringify({ resolvedUrl: resolvedUrl, params: params, query: query, locales: locales })) }) })];
            }
        });
    });
}
exports.router = router;
function jwtVerifier(tokenKey, secretKey, options) {
    var _this = this;
    if (tokenKey === void 0) { tokenKey = 'token'; }
    if (secretKey === void 0) { secretKey = 'JWT_SECRET'; }
    if (options === void 0) { options = {}; }
    return function (_a, next) {
        var _b = _a.req, url = _b.url, cookies = _b.cookies;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var token, jwtPayload, data;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        token = cookies[tokenKey];
                        try {
                            jwtPayload = (0, jsonwebtoken_1.verify)(token, process.env[secretKey], options);
                        }
                        catch (error) {
                            console.error(url, error);
                        }
                        return [4 /*yield*/, next()];
                    case 1:
                        data = _c.sent();
                        if ('props' in data && jwtPayload)
                            data.props['jwtPayload'] = jwtPayload;
                        return [2 /*return*/, data];
                }
            });
        });
    };
}
exports.jwtVerifier = jwtVerifier;
var serverRenderCache = {};
function cache(oneInterval, allInterval) {
    var _this = this;
    if (oneInterval === void 0) { oneInterval = 30 * web_utility_1.Second; }
    if (allInterval === void 0) { allInterval = web_utility_1.Day; }
    function cleanCache() {
        for (var _i = 0, _a = Object.entries(serverRenderCache); _i < _a.length; _i++) {
            var _b = _a[_i], URI = _b[0], expiredAt = _b[1].expiredAt;
            if (Date.now() - expiredAt > allInterval)
                delete serverRenderCache[URI];
        }
    }
    return (function (context, next) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var resolvedUrl, cache, title, data, _a, expiredAt;
        return tslib_1.__generator(this, function (_b) {
            resolvedUrl = context.resolvedUrl;
            cache = (serverRenderCache[resolvedUrl] || (serverRenderCache[resolvedUrl] = {})), title = "[SSR cache] ".concat(resolvedUrl);
            data = cache.data, _a = cache.expiredAt, expiredAt = _a === void 0 ? 0 : _a;
            if ((!data || expiredAt < Date.now()) && !cache.buffer) {
                console.time(title);
                cache.buffer = next()
                    .then(function (data) {
                    cache.data = data;
                    cache.expiredAt = Date.now() + oneInterval;
                    console.log(cache);
                    return data;
                })
                    .finally(function () {
                    delete cache.buffer;
                    cleanCache();
                    console.timeEnd(title);
                });
            }
            return [2 /*return*/, data || cache.buffer];
        });
    }); });
}
exports.cache = cache;
function translator(i18n) {
    var _this = this;
    return function (context, next) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var _a, language, languages;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = context.req.cookies.language, language = _a === void 0 ? '' : _a, languages = (0, mobx_i18n_1.parseLanguageHeader)(context.req.headers['accept-language'] || '');
                    return [4 /*yield*/, i18n.loadLanguages(tslib_1.__spreadArray([language], languages, true).filter(Boolean))];
                case 1:
                    _b.sent();
                    return [2 /*return*/, next()];
            }
        });
    }); };
}
exports.translator = translator;
//# sourceMappingURL=middleware.js.map